package dev.shadowsoffire.apotheosis;

import net.minecraftforge.registries.RegisterEvent;
import java.util.function.BooleanSupplier;
import dev.shadowsoffire.placebo.registry.DeferredHelper;
import net.minecraftforge.fml.event.IModBusEvent;
import net.minecraftforge.fml.loading.FMLPaths;
import net.minecraftforge.network.NetworkRegistry;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.entity.Entity;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.placebo.recipe.NBTIngredient;
import net.minecraft.world.item.alchemy.PotionUtils;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraft.world.item.alchemy.Potion;
import net.minecraft.commands.CommandSourceStack;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import net.minecraft.commands.Commands;
import net.minecraftforge.event.RegisterCommandsEvent;
import net.minecraft.world.entity.player.Player;
import net.minecraftforge.event.entity.player.AttackEntityEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraft.server.packs.resources.PreparableReloadListener;
import dev.shadowsoffire.placebo.util.RunnableReloader;
import net.minecraftforge.event.AddReloadListenerEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.common.crafting.conditions.IConditionSerializer;
import net.minecraftforge.common.crafting.CraftingHelper;
import dev.shadowsoffire.apotheosis.util.ModuleCondition;
import dev.shadowsoffire.apotheosis.advancements.AdvancementTriggers;
import dev.shadowsoffire.apotheosis.adventure.affix.augmenting.RerollResultMessage;
import dev.shadowsoffire.apotheosis.adventure.net.RadialStateChangeMessage;
import dev.shadowsoffire.apotheosis.ench.table.StatsMessage;
import dev.shadowsoffire.apotheosis.ench.table.ClueMessage;
import dev.shadowsoffire.apotheosis.adventure.client.BossSpawnMessage;
import dev.shadowsoffire.placebo.network.MessageProvider;
import dev.shadowsoffire.placebo.network.MessageHelper;
import dev.shadowsoffire.apotheosis.util.ParticleMessage;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.eventbus.api.IEventBus;
import dev.shadowsoffire.apotheosis.compat.PatchouliCompat;
import net.minecraftforge.fml.ModList;
import net.minecraftforge.common.MinecraftForge;
import java.util.function.Consumer;
import net.minecraftforge.eventbus.api.Event;
import dev.shadowsoffire.apotheosis.village.VillageModule;
import dev.shadowsoffire.apotheosis.potion.PotionModule;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.apotheosis.garden.GardenModule;
import dev.shadowsoffire.apotheosis.spawn.SpawnerModule;
import dev.shadowsoffire.apotheosis.ench.EnchModule;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import dev.shadowsoffire.placebo.config.Configuration;
import java.io.File;
import dev.shadowsoffire.placebo.recipe.RecipeHelper;
import net.minecraftforge.network.simple.SimpleChannel;
import net.minecraftforge.fml.common.Mod;

@Mod("apotheosis")
public class Apotheosis
{
    public static final String MODID = "apotheosis";
    public static final SimpleChannel CHANNEL;
    public static final RecipeHelper HELPER;
    public static File configDir;
    public static Configuration config;
    public static boolean enableEnch;
    public static boolean enableAdventure;
    public static boolean enableSpawner;
    public static boolean enablePotion;
    public static boolean enableVillage;
    public static boolean enableGarden;
    public static boolean giveBook;
    private static float localAtkStrength;
    
    public Apotheosis() {
        final IEventBus bus = FMLJavaModLoadingContext.get().getModEventBus();
        if (Apotheosis.enableEnch) {
            bus.register((Object)new EnchModule());
        }
        if (Apotheosis.enableSpawner) {
            bus.register((Object)new SpawnerModule());
        }
        if (Apotheosis.enableGarden) {
            bus.register((Object)new GardenModule());
        }
        if (Apotheosis.enableAdventure) {
            bus.register((Object)new AdventureModule());
        }
        if (Apotheosis.enablePotion) {
            bus.register((Object)new PotionModule());
        }
        if (Apotheosis.enableVillage) {
            bus.register((Object)new VillageModule());
        }
        if (Apotheosis.config.hasChanged()) {
            Apotheosis.config.save();
        }
        bus.post((Event)new ApotheosisConstruction());
        bus.addListener((Consumer)this::init);
        MinecraftForge.EVENT_BUS.addListener((Consumer)this::reloads);
        MinecraftForge.EVENT_BUS.addListener((Consumer)this::trackCooldown);
        MinecraftForge.EVENT_BUS.addListener((Consumer)this::cmds);
        if (ModList.get().isLoaded("patchouli")) {
            PatchouliCompat.register();
        }
        Apoth.RecipeTypes.FLETCHING.getClass();
    }
    
    @SubscribeEvent
    public void init(final FMLCommonSetupEvent e) {
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 0, (MessageProvider)new ParticleMessage.Provider());
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 1, (MessageProvider)new BossSpawnMessage.Provider());
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 2, (MessageProvider)new ClueMessage.Provider());
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 3, (MessageProvider)new StatsMessage.Provider());
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 4, (MessageProvider)new RadialStateChangeMessage.Provider());
        MessageHelper.registerMessage(Apotheosis.CHANNEL, 5, (MessageProvider)new RerollResultMessage.Provider());
        e.enqueueWork(() -> {
            AdvancementTriggers.init();
            CraftingHelper.register((IConditionSerializer)new ModuleCondition.Serializer());
        });
    }
    
    @SubscribeEvent(priority = EventPriority.LOW)
    public void reloads(final AddReloadListenerEvent e) {
        e.addListener((PreparableReloadListener)RunnableReloader.of(() -> MinecraftForge.EVENT_BUS.post((Event)new ApotheosisReloadEvent())));
    }
    
    @SubscribeEvent
    public void trackCooldown(final AttackEntityEvent e) {
        final Player p = e.getEntity();
        Apotheosis.localAtkStrength = p.m_36403_(0.5f);
    }
    
    @SubscribeEvent
    public void cmds(final RegisterCommandsEvent e) {
        final LiteralArgumentBuilder<CommandSourceStack> builder = (LiteralArgumentBuilder<CommandSourceStack>)Commands.m_82127_("apoth");
        MinecraftForge.EVENT_BUS.post((Event)new ApotheosisCommandEvent(builder));
        e.getDispatcher().register((LiteralArgumentBuilder)builder);
    }
    
    public static Ingredient potionIngredient(final Potion type) {
        return (Ingredient)new NBTIngredient(PotionUtils.m_43549_(new ItemStack((ItemLike)Items.f_42589_), type));
    }
    
    public static ResourceLocation loc(final String s) {
        return new ResourceLocation("apotheosis", s);
    }
    
    public static float getLocalAtkStrength(final Entity entity) {
        if (entity instanceof Player) {
            return Apotheosis.localAtkStrength;
        }
        return 1.0f;
    }
    
    public static MutableComponent sysMessageHeader() {
        return Component.m_237110_("[%s] ", new Object[] { Component.m_237113_("Apoth").m_130940_(ChatFormatting.GOLD) });
    }
    
    static {
        CHANNEL = NetworkRegistry.ChannelBuilder.named(new ResourceLocation("apotheosis", "apotheosis")).clientAcceptedVersions(s -> true).serverAcceptedVersions(s -> true).networkProtocolVersion(() -> "1.0.0").simpleChannel();
        HELPER = new RecipeHelper("apotheosis");
        Apotheosis.enableEnch = true;
        Apotheosis.enableAdventure = true;
        Apotheosis.enableSpawner = true;
        Apotheosis.enablePotion = true;
        Apotheosis.enableVillage = true;
        Apotheosis.enableGarden = true;
        Apotheosis.giveBook = true;
        Apotheosis.localAtkStrength = 1.0f;
        Apotheosis.configDir = new File(FMLPaths.CONFIGDIR.get().toFile(), "apotheosis");
        Apotheosis.config = new Configuration(new File(Apotheosis.configDir, "apotheosis.cfg"));
        Apotheosis.enableEnch = Apotheosis.config.getBoolean("Enable Enchantment Module", "general", true, "If the enchantment module is enabled.");
        Apotheosis.enableAdventure = Apotheosis.config.getBoolean("Enable Adventure Module", "general", true, "If the adventure module is loaded.");
        Apotheosis.enableSpawner = Apotheosis.config.getBoolean("Enable Spawner Module", "general", true, "If the spawner module is enabled.");
        Apotheosis.enablePotion = Apotheosis.config.getBoolean("Enable Potion Module", "general", true, "If the potion module is loaded.");
        Apotheosis.enableVillage = Apotheosis.config.getBoolean("Enable Village Module", "general", true, "If the village module is loaded.");
        Apotheosis.enableGarden = Apotheosis.config.getBoolean("Enable Garden Module", "general", true, "If the garden module is loaded.");
        Apotheosis.giveBook = Apotheosis.config.getBoolean("Give Book on First Join", "general", true, "If the Chronicle of Shadows is given to new players.");
        Apotheosis.config.setTitle("Apotheosis Module Control");
        Apotheosis.config.setComment("""
                                     This file allows individual modules of Apotheosis to be enabled or disabled.
                                     Changes will have no effect until the next game restart.
                                     This file must match on client and server.""");
        if (Apotheosis.config.hasChanged()) {
            Apotheosis.config.save();
        }
    }
    
    public static class ApotheosisConstruction extends Event implements IModBusEvent
    {
    }
    
    public static class ApotheosisReloadEvent extends Event
    {
    }
    
    public static class ApotheosisCommandEvent extends Event
    {
        private final LiteralArgumentBuilder<CommandSourceStack> root;
        
        public ApotheosisCommandEvent(final LiteralArgumentBuilder<CommandSourceStack> root) {
            this.root = root;
        }
        
        public LiteralArgumentBuilder<CommandSourceStack> getRoot() {
            return this.root;
        }
    }
    
    public static class ModularDeferredHelper extends DeferredHelper
    {
        protected final BooleanSupplier flag;
        
        public static ModularDeferredHelper create(final BooleanSupplier flag) {
            final ModularDeferredHelper helper = new ModularDeferredHelper(flag);
            FMLJavaModLoadingContext.get().getModEventBus().register((Object)helper);
            return helper;
        }
        
        protected ModularDeferredHelper(final BooleanSupplier flag) {
            super("apotheosis");
            this.flag = flag;
        }
        
        @SubscribeEvent
        public void register(final RegisterEvent e) {
            if (this.flag.getAsBoolean()) {
                super.register(e);
            }
        }
    }
}
